@@ -265,6 +265,67 @@ public:
 	};
 	typedef enum Error Error;
 
+
+#if defined(__GNUC__) || !defined(__hpux)
+#pragma pack(1)
+#endif
+
+	typedef	struct 
+	{
+#if	__BYTE_ORDER == __LITTLE_ENDIAN
+		unsigned char mp_sync1 : 8;
+		unsigned char mp_crc   : 1;
+		unsigned char mp_layer : 2;
+		unsigned char mp_ver   : 2;
+		unsigned char mp_sync2 : 3;
+
+		unsigned char mp_priv  : 1;
+		unsigned char mp_pad   : 1;
+		unsigned char mp_srate : 2;
+		unsigned char mp_brate : 4;
+
+		unsigned char mp_emp   : 2;
+		unsigned char mp_original : 1;
+		unsigned char mp_copyright: 1;
+		unsigned char mp_extend   : 2;
+		unsigned char mp_channels : 2;
+
+#else
+                unsigned char mp_sync1 : 8;
+
+                unsigned char mp_sync2 : 3;
+                unsigned char mp_ver   : 2;
+                unsigned char mp_layer : 2;
+                unsigned char mp_crc   : 1;
+
+                unsigned char mp_brate : 4;
+                unsigned char mp_srate : 2;
+                unsigned char mp_pad   : 1;
+                unsigned char mp_priv  : 1;
+
+                unsigned char mp_channels : 2;
+                unsigned char mp_extend   : 2;
+                unsigned char mp_copyright : 1;
+                unsigned char mp_original : 1;
+                unsigned char mp_emp : 2;
+#endif
+	}	mpeg_audio;
+
+	typedef struct
+	{
+		char tag_id[3];
+		char tag_title[30];
+		char tag_artist[30];
+		char tag_album[30];
+		char tag_year[4];
+		char tag_note[30];
+		unsigned char genre;
+	}	mpeg_tagv1;
+
+#if defined(__GNUC__) || !defined(__hpux)
+#pragma pack()
+#endif
+
 	/**
 	 * Audio source description.
 	 */
@@ -276,7 +337,7 @@ public:
 		unsigned long rate;
 		unsigned long bitrate;
 		unsigned order;
-		unsigned framesize, framecount;
+		unsigned framesize, framecount, headersize;
 		timeout_t framing;
 		char *annotation;
 
@@ -629,6 +690,15 @@ public:
 	 * @param path to codec.
 	 */
 	static bool loadPlugin(const char *path);
+
+	/**
+	 * Maximum framesize for a given coding that may be needed to
+	 * store a result.
+	 *
+	 * @param Info source description object.
+	 * @return maximum possible frame size to allocate for encoded data.
+	 */
+	static size_t maxFramesize(Info &info);
 };
 
 /**
@@ -888,6 +958,7 @@ protected:
 
 	void initialize(void);
 	void getWaveFormat(int size);
+	void mp3info(mpeg_audio *mp3);
 
 	union
 	{
@@ -1076,6 +1147,14 @@ public:
 	 * /
 	time_t getAge(void);
 
+	/**
+	 * Get maximum size of frame buffer for data use.
+	 *
+	 * @return max frame size in bytes.
+	 */
+	inline size_t getSize(void)
+		{return maxFramesize(info);};
+
 	/**
 	 * Close an object associated with an open file.  This
 	 * updates the header metadata with the file length if the
