@@ -575,6 +575,9 @@ void AudioFile::create(const char *name, Info *myinfo, bool exclusive, timeout_t
 		header = getAbsolutePosition();
 		length = getAbsolutePosition();
 		break;
+	case mpeg:
+		framing = 0;
+		info.headersize = 4;	// no crc...
 	case raw:
 		break;
 	}
@@ -676,7 +679,7 @@ void AudioFile::open(const char *name, Mode m, timeout_t framing)
 	unsigned int count;
 	char *ext;
 	unsigned channels;
-	
+	mpeg_audio *mp3 = (mpeg_audio *)&filehdr;	
 	mode = m;
 
 
@@ -867,10 +870,37 @@ void AudioFile::open(const char *name, Mode m, timeout_t framing)
 		}
 		goto done;
 	}
+
+	if(!strnicmp((char *)filehdr, "ID3\003", 4))
+	{
+		afSeek(10);
+		info.order = __BIG_ENDIAN;
+		header = getLong(filehdr + 6) + 10;
+		if(filehdr[5])
+			header += getLong(filehdr + 10);
+		afSeek(header);
+		afRead(filehdr, 4);
+		goto mp3;
+	}
+		
+	if(mp3->mp_sync1 == 0xff && mp3->mp_sync2 == 0x07)
+		goto mp3; 
 	else
 		afSeek(0);
 
+	goto done;
+
+mp3:
+	framing = 0;
+	afSeek(header);
+
+	info.order = __BIG_ENDIAN;
+	info.format = mpeg;
+	mp3info(mp3);
+	return;
+
 done:
+	info.headersize = 0;
 	if(framing)
 		info.setFraming(framing);
 	else
@@ -884,6 +914,149 @@ done:
 	}
 }
 
+void AudioFile::mp3info(mpeg_audio *mp3)
+{
+	info.headersize = 4;
+
+	switch(mp3->mp_layer)
+	{
+	case 0x03:
+		info.encoding = mp1Audio;
+		break;
+	case 0x02:
+		info.encoding = mp2Audio;
+		break;
+	case 0x01:
+		info.encoding = mp3Audio;
+		break;
+	}
+
+	switch(mp3->mp_ver)
+	{
+	case 0x03:
+		info.bitrate = 32000;
+		switch(mp3->mp_srate)
+		{
+		case 00:
+			info.rate = 44100;
+			break;
+		case 01:
+			info.rate = 48000;
+			break;
+		case 02:
+			info.rate = 32000;
+		}
+		switch(mp3->mp_layer)
+		{
+		case 0x03:
+			info.bitrate = 32000 * mp3->mp_brate;
+			break;
+		case 0x02:
+			if(mp3->mp_brate < 8)
+				info.bitrate = 16000 * (mp3->mp_brate + 1);
+			else	
+				info.bitrate = 32000 * (mp3->mp_brate - 4); 
+			break;
+		case 0x01:
+			switch(mp3->mp_brate)
+			{
+			case 0x02:
+				info.bitrate = 40000;
+				break;
+			case 0x03:
+				info.bitrate = 48000;
+				break;
+			case 0x04:
+				info.bitrate = 56000;
+				break;
+			case 0x05:
+				info.bitrate = 64000;
+				break;
+			case 0x06:
+				info.bitrate = 80000;
+				break;
+			case 0x07:
+				info.bitrate = 96000;
+				break;
+			case 0x08:
+				info.bitrate = 112000;
+				break;
+			case 0x09:
+				info.bitrate = 128000;
+				break;
+			case 0x0a:
+				info.bitrate = 160000;
+				break;
+			case 0x0b:
+				info.bitrate = 192000;
+				break;
+			case 0x0c:
+				info.bitrate = 224000;
+				break;
+			case 0x0d:
+				info.bitrate = 256000;
+				break;
+			case 0x0e:
+				info.bitrate = 320000;
+				break;
+			}
+			break;	
+		}
+		break;
+	case 0x00:
+		switch(mp3->mp_srate)
+		{
+		case 00:
+			info.rate = 11025;
+			break;
+		case 01:
+			info.rate = 12000;
+			break;
+		case 02:
+			info.rate = 8000;
+			break;
+		}
+	case 0x02:
+		if(mp3->mp_ver == 0x02)
+		{
+			switch(mp3->mp_srate)
+			{
+			case 00:
+				info.rate = 22050;
+				break;
+			case 01:
+				info.rate = 24000;
+				break;
+			case 02:
+				info.rate = 16000;
+				break;
+			}
+		}
+		switch(mp3->mp_layer)
+		{
+		case 0x03:
+			if(mp3->mp_brate < 0x0d)
+				info.bitrate = 16000 * (mp3->mp_brate + 1);
+			else if(mp3->mp_brate == 0x0d)
+				info.bitrate = 224000;
+			else
+				info.bitrate = 256000;
+			break;
+		case 0x02:
+		case 0x01:
+			if(mp3->mp_brate < 9)
+				info.bitrate = 8000 * mp3->mp_brate;
+			else
+				info.bitrate = 16000 * (mp3->mp_brate - 4);
+		}
+	}
+
+	if(mp3->mp_crc)
+		info.headersize = 6;
+
+	info.set();
+}
+
 Audio::Info::Info()
 {
 	clear();
@@ -931,6 +1104,19 @@ void Audio::Info::setFraming(timeout_t timeout)
 
 void Audio::Info::set(void)
 {
+
+	switch(encoding)
+	{
+	case mp1Audio:
+		framecount = 384;
+		framesize = (12 * bitrate / rate) * 4 + headersize;
+		return;
+	case mp2Audio:
+	case mp3Audio:
+		framecount = 1152;
+		framesize = (144 * bitrate / rate) + headersize;
+		return;
+	}
 	if(!framesize)
 		framesize = getFrame(encoding);
 
@@ -1132,7 +1318,28 @@ ssize_t AudioFile::getBuffer(Encoded addr, size_t bytes)
 	char *fname;
 	int count;
 	unsigned long curpos, xfer = 0;
+	mpeg_audio *mp3 = (mpeg_audio *)addr;
 
+	if(!bytes && info.format == mpeg)
+	{
+rescan:
+		count = afRead(addr, 4);
+		if(count < 0)
+			return count;
+		if(count < 4)
+			return 0;
+		if(mp3->mp_sync1 != 0xff || mp3->mp_sync2 != 0x07)
+		{
+			afSeek(getAbsolutePosition() - 3);
+			goto rescan;
+		}	
+		mp3info(mp3);	
+		count = afRead(addr + 4, info.framesize - 4);
+		if(count > 0)
+			count += 4;
+		return count;
+	}
+			
 	if(!bytes)
 		bytes = info.framesize;
 
@@ -1260,6 +1467,13 @@ ssize_t AudioFile::putBuffer(Encoded addr, size_t len)
 {
 	int count;
 	unsigned long curpos;
+	mpeg_audio *mp3 = (mpeg_audio *)addr;
+
+	if(!len && info.format == mpeg)
+	{
+		mp3info(mp3);
+		len = info.framesize;
+	}
 
 	if(!len)
 		len = info.framesize;
