@@ -219,7 +219,7 @@ void AudioStream::create(const char *fname, Info *info, bool exclusive, timeout_
 		streamable = false;
 		return;
 	}
-	framebuf = new unsigned char[AudioFile::info.framesize];
+	framebuf = new unsigned char[maxFramesize(AudioFile::info)];
 }
 
 void AudioStream::open(const char *fname, Mode m, timeout_t framing)
@@ -241,7 +241,7 @@ void AudioStream::open(const char *fname, Mode m, timeout_t framing)
 	if(!codec)
 		streamable = false;
 	else
-		framebuf = new unsigned char[info.framesize];
+		framebuf = new unsigned char[maxFramesize(info)];
 }
 
 unsigned AudioStream::getCount(void)
